Package: alabaster.sce
Title: Load and Save SingleCellExperiment from File
Version: 1.10.0
Date: 2024-10-16
Authors@R: person("Aaron", "Lun", role=c("aut", "cre"), email="infinite.monkeys.with.keyboards@gmail.com")
License: MIT + file LICENSE
Description: 
    Save SingleCellExperiment into file artifacts, and load them back into memory.
    This is a more portable alternative to serialization of such objects into RDS files.
    Each artifact is associated with metadata for further interpretation;
    downstream applications can enrich this metadata with context-specific properties.
Depends: SingleCellExperiment, alabaster.base
Imports: methods, alabaster.se, jsonlite
Suggests: knitr, testthat, BiocStyle, rmarkdown
VignetteBuilder: knitr
RoxygenNote: 7.2.3
biocViews: DataImport, DataRepresentation
git_url: https://git.bioconductor.org/packages/alabaster.sce
git_branch: RELEASE_3_22
git_last_commit: a6eed09
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 02:28:44 UTC; biocbuild
Author: Aaron Lun [aut, cre]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
